import { Fragment } from 'wp.element';
import PropTypes from 'prop-types';
import { compose } from 'wp.compose';
import { __ } from 'wp.i18n';
import {
  InspectorControls,
  MediaUpload,
  PanelColorSettings,
} from 'wp.blockEditor';
import {
  IconButton,
  RangeControl,
  SelectControl,
  PanelBody,
} from 'wp.components';
import { withSelect } from 'wp.data';
import startCase from 'lodash.startcase';
import get from 'lodash.get';

import ImagePlaceholder from '../../components/image-placeholder/ImagePlaceholder';
import MarginControls from '../../components/controls/margin-controls';
import ResponsiveControl from '../../components/controls/responsive-control/ResponsiveControl';
import useUniqueId from '../../hooks/useUniqueId';
import getBlockId from '../../util/getBlockId';
import ImageComparisonStyle from './style';

const propTypes = {
  attributes: PropTypes.shape({
    uniaueId: PropTypes.string,
    idA: PropTypes.number,
    urlA: PropTypes.string,
    idB: PropTypes.number,
    urlB: PropTypes.string,
    offset: PropTypes.number,
    blockMargin: PropTypes.object,
    blockPadding: PropTypes.object,
    backgroundColor: PropTypes.string,
    imageSize: PropTypes.string,
  }),
  className: PropTypes.string.isRequired,
  setAttributes: PropTypes.func.isRequired,
  isSelected: PropTypes.bool.isRequired,
  imageA: PropTypes.shape({
    media_details: PropTypes.shape({
      sizes: PropTypes.object,
    }),
  }),
  imageB: PropTypes.shape({
    media_details: PropTypes.shape({
      sizes: PropTypes.object,
    }),
  }),
  clientId: PropTypes.string.isRequired,
};

const ImageComparisonEdit = ({
  className,
  attributes,
  setAttributes,
  isSelected,
  imageA,
  imageB,
  clientId,
}) => {
  const {
    uniqueId,
    urlA,
    idA,
    urlB,
    idB,
    offset,
    backgroundColor,
    imageSize,
  } = attributes;
  const availableSizes = get(imageA, 'media_details.sizes');

  useUniqueId({ attributes, setAttributes, clientId });

  const onSetImageSize = size => {
    const imageASizes = get(imageA, 'media_details.sizes');
    const imageBSizes = get(imageB, 'media_details.sizes');
    const newUrlA = get(imageASizes[size], 'source_url', urlA);
    const newUrlB = get(imageBSizes[size], 'source_url', urlB);

    setAttributes({
      imageSize: size,
      urlA: newUrlA,
      urlB: newUrlB,
    });
  };

  const getImageAttributes = image => {
    const url = get(image, `sizes[${imageSize}].url`, image.url);

    return { url, id: image.id };
  };

  const blockId = getBlockId(uniqueId);

  return (
    <Fragment>
      <div
        id={blockId}
        className={className}
        style={{
          backgroundColor: backgroundColor || undefined,
        }}
      >
        <ImageComparisonStyle attributes={attributes} />
        <div className={`${className}-pane`}>
          {urlA && idA ? (
            <figure className={`${className}-figure`}>
              <img src={urlA} alt="" />

              <MediaUpload
                onSelect={image => {
                  const { id, url } = getImageAttributes(image);

                  setAttributes({
                    idA: id,
                    urlA: url,
                  });
                }}
                allowedTypes={['image']}
                value={idA}
                render={({ open }) => (
                  <IconButton
                    className="components-toolbar__control"
                    label={__('Edit Image')}
                    icon="format-image"
                    onClick={open}
                  />
                )}
              />
            </figure>
          ) : (
            <ImagePlaceholder
              icon="format-image"
              label={__('Image A')}
              onSelectImage={image => {
                const { id, url } = getImageAttributes(image);

                setAttributes({
                  idA: id,
                  urlA: url,
                });
              }}
            />
          )}
        </div>

        <div className={`${className}-pane`}>
          {urlB && idB ? (
            <figure className={`${className}-figure`}>
              <img src={urlB} alt="" />

              <MediaUpload
                onSelect={image => {
                  const { id, url } = getImageAttributes(image);

                  setAttributes({
                    idB: id,
                    urlB: url,
                  });
                }}
                allowedTypes={['image']}
                value={idB}
                render={({ open }) => (
                  <IconButton
                    className="components-toolbar__control"
                    label={__('Edit Image')}
                    icon="format-image"
                    onClick={open}
                  />
                )}
              />
            </figure>
          ) : (
            <ImagePlaceholder
              icon="format-image"
              label={__('Image B')}
              onSelectImage={image => {
                const { id, url } = getImageAttributes(image);

                setAttributes({
                  idB: id,
                  urlB: url,
                });
              }}
            />
          )}
        </div>
      </div>

      {isSelected && (
        <InspectorControls>
          <PanelBody title={__('Settings')} initialOpen>
            <RangeControl
              label={__('Default Offset')}
              min={1}
              max={100}
              value={offset}
              onChange={value => setAttributes({ offset: value })}
            />

            {imageA && imageB && availableSizes && (
              <SelectControl
                label={__('Source Image Size')}
                value={imageSize}
                options={Object.keys(availableSizes).map(name => {
                  const size = availableSizes[name];

                  return {
                    value: name,
                    label: `${startCase(name)} - ${size.width}×${size.height}`,
                  };
                })}
                onChange={onSetImageSize}
              />
            )}
          </PanelBody>

          <PanelColorSettings
            title={__('Block Appearance')}
            initialOpen={false}
            colorSettings={[
              {
                value: backgroundColor,
                onChange: value => setAttributes({ backgroundColor: value }),
                label: __('Background Color'),
              },
            ]}
            onChange={value => setAttributes({ backgroundColor: value })}
          >
            <ResponsiveControl>
              {breakpoint => (
                <MarginControls
                  label={__('Padding (px)')}
                  attributeKey="blockPadding"
                  attributes={attributes}
                  setAttributes={setAttributes}
                  breakpoint={breakpoint}
                />
              )}
            </ResponsiveControl>

            <ResponsiveControl>
              {breakpoint => (
                <MarginControls
                  label={__('Margin (px)')}
                  attributeKey="blockMargin"
                  attributes={attributes}
                  setAttributes={setAttributes}
                  breakpoint={breakpoint}
                />
              )}
            </ResponsiveControl>
          </PanelColorSettings>
        </InspectorControls>
      )}
    </Fragment>
  );
};

ImageComparisonEdit.propTypes = propTypes;

export default compose([
  withSelect((select, props) => {
    const { getMedia } = select('core');
    const { idA, idB } = props.attributes;

    return {
      imageA: idA ? getMedia(idA) : null,
      imageB: idB ? getMedia(idB) : null,
    };
  }),
])(ImageComparisonEdit);
